from webob import Request
from webob.multidict import NestedMultiDict
from tw2.core.testbase import assert_in_xml, assert_eq_xml, WidgetTest
from nose.tools import raises
from cStringIO import StringIO
from tw2.core import EmptyField, IntValidator, ValidationError
from cgi import FieldStorage
import formencode

import webob
if hasattr(webob, 'NestedMultiDict'):
    from webob import NestedMultiDict
else:
    from webob.multidict import NestedMultiDict

import tw2.protovis.widgets as w

# TODO -- test 'em all!



"""
TODO
"""

import tw2.core as twc
import tw2.protovis.core as twp
from tw2.protovis.core import pv

class js(twc.JSSymbol):
    def __init__(self, src):
        super(js, self).__init__(src=src)

class SparkLine(twp.PVWidget):
    """
    A sparkline is a word-sized data visualization, allowing visual
    representations of data to be embedded directly in prose. This avoids
    any interruption in the flow of text, with the data presented in
    descriptive context. Sparklines can be used where space is limited,
    unlike standalone figures.
    """
    p_dots = twc.Param('dots', default=False)

    def prepare(self):
        self.init_js = js(
            """
            var dots = %i;
            var data = %s;
            var n = data.length;
            var w = n,
                h = %i,
                min = pv.min.index(data),
                max = pv.max.index(data);
            """ % (self.p_dots, self.p_data, self.p_height))
       
        self.p_width = len(self.p_data)
        self.setupRootPanel()
        self.margin(2)

        self.add(pv.Line) \
          .data(js('data')) \
          .left(js('pv.Scale.linear(0, n - 1).range(0, w).by(pv.index)')) \
          .bottom(js('pv.Scale.linear(data).range(0, h)')) \
          .strokeStyle("#000") \
          .lineWidth(1) \
        .add(pv.Dot) \
          .visible(
              js('function() (dots && this.index==0) || this.index==n - 1')) \
          .strokeStyle(None) \
          .fillStyle("brown") \
          .radius(2) \
        .add(pv.Dot) \
          .visible(
              js('function() dots && (this.index==min || this.index==max)')) \
          .fillStyle("steelblue");

class SparkBar(twp.PVWidget):
    """ A sparkbar is just like a sparkline, but,... you know. """
    def prepare(self):
        self.init_js = js(
            """
            var data = %s;
            var n = data.length;
            var w = n,
                h = %i;
            """ % (self.p_data, self.p_height))
       
        self.setupRootPanel()

        self.add(pv.Bar) \
          .data(js('data'))\
          .width(4)\
          .left(js('function() 5 * this.index'))\
          .height(js('function(d) Math.round(h * d)'))\
          .bottom(0)

class StreamGraph(twp.PVWidget):
    """
    Streamgraphs are a generalization of stacked area graphs where the
    baseline is free. By shifting the baseline, it is possible to minimize
    the change in slope (or "wiggle") in individual series, thereby making
    it easier to perceive the thickness of any given layer across the data.
    Byron & Wattenberg describe several streamgraph algorithms in "Stacked
    Graphs--Geometry & Aesthetics", several of which are implemented by
    pv.Layout.Stack.
    """
    def prepare(self):
        self.init_js = js(
            """
            var n = 20, m = 400;
            var data = %s,
                w = %i,
                h = %i,
                x = pv.Scale.linear(0, m - 1).range(0, w),
                y = pv.Scale.linear(0, 2 * n).range(0, h);
            """ % (self.p_data, self.p_width, self.p_height))
        
        self.setupRootPanel()

        self.add(pv.Layout.Stack)\
                .layers(js('data'))\
                .order('inside-out')\
                .offset('wiggle')\
                .x(js('x.by(pv.index)'))\
                .y(js('y'))\
              .layer.add(pv.Area)\
                .fillStyle(js('pv.ramp("#aad", "#556").by(Math.random)'))\
                .strokeStyle(js('function() this.fillStyle().alpha(0.5)'))

class BubbleChart(twp.PVWidget):
    """
    Bubble charts encode data in the area of circles. Although less
    perceptually accurate than bar charts, they can pack hundreds of
    values into a small space. A similar technique is the Dorling
    cartogram, where circles are positioned according to geography rather
    than arbitrarily. Here we compare the file sizes of the Flare
    visualization toolkit.
    """
    def prepare(self):
        self.init_js = js(
            """
            var data = pv.nodes(%s);
            var format = pv.Format.number();
            """ % (self.p_data))
        
        self.setupRootPanel()

        self.add(pv.Layout.Pack) \
            .top(-50) \
            .bottom(-50) \
            .nodes(js('data')) \
            .size(js('function(d) d.nodeValue.value')) \
            .spacing(0) \
            .order(None) \
          .node.add(pv.Dot) \
            .fillStyle(
        js('pv.Colors.category20().by(function(d) d.nodeValue.group)')) \
            .strokeStyle(js('function() this.fillStyle().darker()')) \
            .visible(js('function(d) d.parentNode')) \
            .title(js('function(d) d.nodeValue.name + ": " + format(d.nodeValue.value)')) \
          .anchor("center").add(pv.Label) \
            .text(js('function(d) d.nodeValue.text'))



""" Samples of how to use tw2.protovis.custom

Each class exposed in the widgets submodule has an accompanying Demo<class>
widget here with some parameters filled out.

The demos implemented here are what is displayed in the tw2.devtools
WidgetBrowser.
"""
from widgets import (
    SparkLine,
    SparkBar,
    StreamGraph,
    BubbleChart,
)
from widgets import js
from tw2.core import JSSymbol

import math
import random
import os

class DemoSparkLine(SparkLine):
    p_height = 10
    p_data = [
          40, 115, 100, 80, 60, 40, 23, 10, 10, 25, 75, 145,
          130, 130, 80, 65, 20, 10, 5, 10, 60, 190, 180, 175,
          120, 50, 35, 20, 10, 15, 30, 60, 105, 105, 105, 80, 65
    ]

class DemoSparkBar(SparkBar):
    p_height = 10
    p_data = [0.2, 0.3, 0.6, 0.1, 0.9, 0.8, 0.23, 0.77, 0.63, 0.43, 0.59, 0.11]

# The following are some data generation functions used by the streamgraph demo
def waves(n, m):
    def f(i, j):
        x = 20 * j / m - i / 3
        if x > 0:
            return 2 * x * math.exp(x * -0.5)
        return 0
    return map(lambda i : map(lambda j : f(i, j), range(m)), range(n))

def layers(n, m):
    def bump(a):
        x = 1.0 / (.1 + random.random())
        y = 2.0 * random.random() - 0.5
        z = 10.0 / (0.1 + random.random())
        for i in range(m):
            w = (float(i) / m - y) * z
            a[i] += x * math.exp(-w * w)
        return a
    def f(*args):
        a = [0] * m
        for i in range(5):
            a = bump(a)
        return a
    return map(f, range(n))

class DemoStreamGraph(StreamGraph):
    def prepare(self):
        self.p_data = layers(20, 400)
        super(DemoStreamGraph, self).prepare()

class DemoBubbleChart(BubbleChart):
    p_height = 750
    p_width = 750
    def prepare(self):
        self.p_data = []
        for tup in os.walk('.'):
            dir, dirs, files = tup
            for file in files:
                if (file.endswith('pyc') 
                    or file.endswith('pyo')
                    or file.endswith('.swp')):
                    continue
                nodename = "%s/%s" % (dir, file)
                value = int(os.path.getsize(nodename)) + 1
                clip = int(math.sqrt(value)/50.0) # trim text based on value
                self.p_data.append({
                    'name' : nodename,
                    'value' : value,
                    'text' : nodename.split('/')[-1][:clip],
                    'group' : "/".join(nodename.split('/')[:-1]),
                })
        super(DemoBubbleChart, self).prepare()




"""
A bunch of cool widgets from protovis (under the 'custom' heading in their
examples page).

Get this source from http://github.com/ralphbean/tw2.protovis.custom

Protovis itself can be found here:  http://vis.stanford.edu/protovis/
"""

from widgets import *
